"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const types_1 = require("../types");
const price_1 = require("@melonproject/token-math/price");
const bigInteger_1 = require("@melonproject/token-math/bigInteger");
exports.initializeOrderbook = (options) => ({
    quote: options.pair.quote,
    base: options.pair.quote,
    network: options.network,
    bids: [],
    asks: [],
});
const sortOrders = (a, b) => {
    const priceA = price_1.toAtomic(a.trade);
    const priceB = price_1.toAtomic(b.trade);
    const difference = parseFloat(bigInteger_1.subtract(priceB, priceA).toString());
    // Sort by volumes if prices are identical.
    if (difference === 0) {
        const quantityA = a.trade.base.quantity;
        const quantityB = b.trade.base.quantity;
        return parseFloat(bigInteger_1.subtract(quantityB, quantityA).toString());
    }
    return difference;
};
exports.aggregateOrderbookFromOrders = (carry, current) => {
    const bids = current.filter(order => order.type === types_1.OrderType.BID);
    const asks = current.filter(order => order.type === types_1.OrderType.ASK);
    return Object.assign({}, carry, { bids: carry.bids.concat(bids).sort(sortOrders), asks: carry.asks.concat(asks).sort(sortOrders) });
};
exports.aggregateOrderbookFromEvents = (carry, current) => {
    if (current.event === types_1.NormalizedMessageType.SNAPSHOT) {
        const snapshot = current;
        const bids = snapshot.orders.filter(order => order.type === types_1.OrderType.BID);
        const asks = snapshot.orders.filter(order => order.type === types_1.OrderType.ASK);
        const { exchange } = current;
        return Object.assign({}, carry, { bids: carry.bids
                .filter(item => item.exchange !== exchange)
                .concat(bids)
                .sort(sortOrders), asks: carry.asks
                .filter(item => item.exchange !== exchange)
                .concat(asks)
                .sort(sortOrders) });
    }
    if (current.event === types_1.NormalizedMessageType.ADD) {
        const add = current;
        const { id, order } = add;
        return Object.assign({}, carry, { asks: order.type === types_1.OrderType.ASK
                ? carry.asks
                    .filter(item => item.id !== id)
                    .concat([order])
                    .sort(sortOrders)
                : carry.asks, bids: order.type === types_1.OrderType.BID
                ? carry.bids
                    .filter(item => item.id !== id)
                    .concat([order])
                    .sort(sortOrders)
                : carry.bids });
    }
    if (current.event === types_1.NormalizedMessageType.REMOVE) {
        const remove = current;
        const { id } = remove;
        return Object.assign({}, carry, { asks: carry.asks.filter(order => order.id !== id), bids: carry.bids.filter(order => order.id !== id) });
    }
    return carry;
};
