"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const R = __importStar(require("ramda"));
const Rx = __importStar(require("rxjs"));
const axios_1 = __importDefault(require("axios"));
const operators_1 = require("rxjs/operators");
const webSocket_1 = require("rxjs/webSocket");
const isomorphic_ws_1 = __importDefault(require("isomorphic-ws"));
const types_1 = require("../../types");
const price_1 = require("@melonproject/token-math/price");
const quantity_1 = require("@melonproject/token-math/quantity");
const debug_1 = require("../debug");
const debug = require('debug')('exchange-aggregator:ethfinex');
const wethToEth = (token) => token.replace(/^WETH$/, 'ETH');
const subscribeMessage = (options) => {
    const base = wethToEth(options.pair.base.symbol);
    const quote = wethToEth(options.pair.quote.symbol);
    const message = {
        event: 'subscribe',
        channel: 'book',
        symbol: `t${base}${quote}`,
        prec: 'R0',
    };
    return message;
};
const getWebsocketUrl = (options) => {
    switch (options.network) {
        case types_1.Network.MAINNET:
            return 'wss://api.ethfinex.com/ws/2';
        default:
            throw new Error('Ethfinex only supports the MAINNET network.');
    }
};
const getHttpUrl = (options) => {
    const base = wethToEth(options.pair.base.symbol);
    const quote = wethToEth(options.pair.quote.symbol);
    switch (options.network) {
        case types_1.Network.MAINNET:
            return `https://api.ethfinex.com/v2/book/t${base}${quote}/R0`;
        default:
            throw new Error('Ethfinex only supports the MAINNET network.');
    }
};
const normalizeOrder = (options, order) => {
    const [id, price, amount] = order;
    const key = `${types_1.Exchange.ETHFINEX}:${id}`;
    const volume = Math.abs(amount);
    return {
        id: Buffer.from(key).toString('base64'),
        type: amount > 0 ? types_1.OrderType.BID : types_1.OrderType.ASK,
        exchange: types_1.Exchange.ETHFINEX,
        trade: price_1.createPrice(quantity_1.createQuantity(options.pair.base, volume), quantity_1.createQuantity(options.pair.quote, price * volume)),
    };
};
const normalizeOrderEvent = (options, order) => {
    const [, price] = order;
    const normalized = normalizeOrder(options, order);
    const event = price === 0 ? types_1.NormalizedMessageType.REMOVE : types_1.NormalizedMessageType.ADD;
    return {
        id: normalized.id,
        event,
        exchange: types_1.Exchange.ETHFINEX,
        order: normalized,
    };
};
const normalizeSnapshotEvent = (options, orders) => {
    const processed = orders
        .filter(([, price]) => price !== 0)
        .map(order => normalizeOrder(options, order));
    return {
        event: types_1.NormalizedMessageType.SNAPSHOT,
        exchange: types_1.Exchange.ETHFINEX,
        orders: processed,
    };
};
exports.fetchEthfinexOrders = (options) => __awaiter(this, void 0, void 0, function* () {
    const { data } = yield axios_1.default.get(getHttpUrl(options));
    return data.map((order) => normalizeOrder(options, order));
});
exports.observeEthfinex = (options) => {
    try {
        const open$ = new Rx.Subject();
        const close$ = new Rx.Subject();
        const url = getWebsocketUrl(options);
        const ws$ = webSocket_1.webSocket({
            WebSocketCtor: isomorphic_ws_1.default,
            closeObserver: close$,
            openObserver: open$,
            url,
        });
        ws$.next(subscribeMessage(options));
        open$.subscribe(() => {
            debug('Opening connection.');
        });
        close$.subscribe(() => {
            debug('Closing connection.');
        });
        const messages$ = ws$.pipe(operators_1.filter(R.is(Array)), operators_1.map(R.nth(1)));
        const snapshots$ = messages$.pipe(operators_1.filter(R.compose(R.is(Array), R.head)), operators_1.map(orders => normalizeSnapshotEvent(options, orders)));
        const updates$ = messages$.pipe(operators_1.filter(R.compose(R.is(Number), R.head)), operators_1.map(order => normalizeOrderEvent(options, order)));
        return Rx.merge(snapshots$, updates$).pipe(operators_1.tap(value => {
            debug(...debug_1.debugEvent(value));
        }));
    }
    catch (error) {
        return Rx.throwError(error);
    }
};
